'use strict';

var gulp = require('gulp-help')(require('gulp'));
var runSequence = require('run-sequence');
var _ = require('lodash');

var finalTasks = {
    bundle: []
};

function run(settings) {

    _.forEach(settings, function(options, configTaskName) {
        loadTask(configTaskName, settings);
    });

    return createTasks();
}

function loadTask(configTaskName, settings) {
    var enableInBundle = _enableInBundle(configTaskName);
    var taskName = _parseTaskName(configTaskName);
    var taskOptions = settings[configTaskName];
    var task =  require('./tasks/' + taskName)(taskOptions);

    finalTasks[taskName] = {
        desc: task.desc,
        subtasks: []
    };

    if (enableInBundle) {
        finalTasks.bundle.push(taskName);
    }

    task.subtasks.forEach(function(subtask){
        var enableSubtask = _enbleSubtask(subtask, taskOptions);

        if (enableSubtask) {
            subtask = _parseSubtaskOptions(subtask);
            finalTasks[taskName].subtasks.push(subtask);
        }
    });
}

function createTasks() {

    var bundle = finalTasks.bundle;
    delete finalTasks.bundle;

    _.forEach(finalTasks, function(task, taskName) {
        if (task.subtasks.length) {
            var selectedSubtasks = [];

            task.subtasks.forEach(function(subtask) {
                var gulpTaskName = [taskName, subtask.name].join(':');
                selectedSubtasks.push(subtask.name);
                gulp.task(gulpTaskName, subtask.desc, subtask.callback);
            });

            gulp.task(taskName, task.desc, function(done) {
                var sequenceTasks = selectedSubtasks.map(function(item){
                  return taskName + ':' + item;
                });

                sequenceTasks.push(done);
                runSequence.apply({}, sequenceTasks);
            });
        }
    });

    if (bundle.length) {
        gulp.task('build', 'Run tasks: ' + bundle.join(', '), bundle);
    }

}

function _enableInBundle(name) {
    return name.indexOf('_');
}

function _parseTaskName(name) {
    if (!name.indexOf('_')) {
        name = name.substr(1);
    }

    return name;
}

function _enbleSubtask(subtask, options) {
    if ((subtask.forceRun !== undefined && subtask.forceRun) || options._tasks === undefined) {
        return true;
    } else if (options._tasks.type === "enable") {
        if (options._tasks.enable.indexOf(subtask.name) > -1) {
            return true;
        } else {
            return false;
        }
    } else if (options._tasks.type === "disable") {
        if (options._tasks.disable.indexOf(subtask.name) > -1) {
            return false;
        } else {
            return true;
        }
    }
}

function _parseSubtaskOptions(task) {
    task.desc = task.desc || false;
    return task;
}

/////////////////////////////

module.exports = run;
